package MN::Theme; # class to manage MyNews themes

use strict;

@MN::Theme::ISA = qw(MN::Object);

use MN::Object;
use HTTP::Lite;
use Archive::Tar;

my %current = ();
my %updates = ();

sub getCurrent # get current theme hash
{
    return \%current;
}

sub setCurrent # put the theme in the current hash table
{
    my $self = shift;
    $current{$self->{id}} = $self;
}

sub new # constructor
{
    my $class_name = shift;

    my $self = new MN::Object();
    
    bless $self, $class_name;

    return $self; # new Theme object
}
sub getClass # get database table for the class 
{
    return "mntheme"; # "mntheme"
}

sub getKey # get database key for the class
{
    return "id"; # "id"
}

sub getFields # get database fields for the class
{
    return { # fields hash table
	"id" => "string",
	"name" => "string",
	"comment" => "string",
	"mnid" => "string",
	"mntm" => "number",
    };
}

sub getTheme # get news theme by its name
{
    my $self = shift;
    my $name = shift; # name of the theme

    if (!$name)
    {
	return undef;
    }

    if (!$current{$name})
    {
	$current{$name} = MN::Theme->getObject($name);
    }
    return $current{$name};
}

sub delete # database destructor
{
    my $self = shift;

    # delete theme files
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{webdir})
    {
	my $thdir = "$mne->{conf}->{webdir}/css/themes/$self->{id}";
	opendir(DIR, $thdir);
	my $file;
	while ($file = readdir(DIR))
	{
	    if ($file =~ /^\.+$/) {next;}
	    unlink("$thdir/$file");
	}
	closedir(DIR);
	rmdir($thdir);
    }

    $self->SUPER::delete();
}

sub getSelectorCode # return HTML code for the theme selector
{
    my $self = shift;
    my $selected = shift; # optional default selected theme
    my $submit = shift; # 1 to submit on value change
    my $css = shift; # optional css class prefix
    my $name = shift || "theme"; # optional selector name ("theme" by default)

    my $file;
    my $theme;
    my $code = '<select class="'.$css.'formselect" name="'.$name.'"';
    if ($submit)
    {
	$code .= ' onchange="form.submit();"';
    }
    $code .= '>'."\n";
    $code .= '<option value=""';
    if (!$selected)
    {
	$code .= " selected";
    }
    $code .= ">\n";

    foreach $theme ($self->getObjects("", "name"))
    {
	$code .= '<option value="'.$theme->{id}.'" title="'.$theme->{comment}.'"';
	if ($theme->{id} eq $selected)
	{
	    $code .= " selected";
	}
	$code .= ">$theme->{name}\n";
    }
    $code .= "</select>\n";

    return $code;
}

sub getLastUpdate # get the date of the last theme update from MyNews website
{
    my $self = shift;

    my $date = "";

    if ($self->{mntm})
    {
	my @tm = localtime($self->{mntm});
	if ($::MN::CGI::user && $::MN::CGI::user->{lang} eq "fr")
	{
	    $date = sprintf("%02d/%02d/%04d %02d:%02d:%02d", $tm[3], $tm[4]+1, $tm[5]+1900, $tm[2], $tm[1], $tm[0]);
	}
	else
	{
	    $date = sprintf("%02d/%02d/%04d %02d:%02d:%02d", $tm[4]+1, $tm[3], $tm[5]+1900, $tm[2], $tm[1], $tm[0]);
	}
	
    }
    return $date;
}

sub readCSS # read CSS file of theme and put if in css field
{
    my $self = shift;
    my $mode = shift; # 1 for printer mode

    if (!$self->{id}) { return ; }
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    my $cssfile;

    if ($mode)
    {
	$cssfile = "$mne->{conf}->{webdir}/css/themes/$self->{id}/print.css";
    }
    else
    {
	$cssfile = "$mne->{conf}->{webdir}/css/themes/$self->{id}/theme.css";
    }
    open(CSS, "$cssfile");
    my $sep = "";
    my $line;
    $self->{css} = "";
    while ($line = <CSS>)
    {
	$line =~ s/[\r\n]+$//;
	$self->{css} .= $sep.$line;
	$sep = "\n";
    }
    close(CSS);
}

sub writeCSS # write CSS file with the content of the css field of theme
{
    my $self = shift;
    my $mode = shift; # 1 for printer mode

    if (!$self->{id}) { return ; }
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    my $thdir = "$mne->{conf}->{webdir}/css/themes/$self->{id}";
    if (!-e $thdir)
    {
	mkdir($thdir, 0750);
	chmod(0750, $thdir);
    }
    my $cssfile;
    if ($mode)
    {
	$cssfile = "$thdir/print.css";
    }
    else
    {
	$cssfile = "$thdir/theme.css";
    }
    if ($self->{css})
    {
	open(CSS, ">$cssfile");
	print CSS $self->{css};
	close(CSS);
    }
    else
    {
	unlink($cssfile);
    }
}

sub getMyNewsUpdates # get themes update time from MyNews website
{
    my $self = shift;

    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{mnurl})
    {
	my $http = new HTTP::Lite();
	if ($ENV{REMOTE_USER})
	{
	    $http->add_req_header("Accept-Language", MN::CGI->getUser()->{lang});
	}
	$http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	my $req = $http->request("$mne->{conf}->{mnurl}/themes/update.php");
	if ($req)
	{
	    my $line;
	    foreach $line (split(/\n/,$http->body()))
	    {
		if ($line =~ /^(\S+)\s*(\d+)$/)
		{
		    $updates{$1} = $2;
		}
	    }
	}
    }
}

sub getUpdateCode # get HTML code for the theme status
{
    my $self = shift;
    my $css = shift;

    my $val;
    my $code = "";
    my $msg;

    if ($self->{mnid} && $updates{$self->{mnid}})
    {
	my $mne = MN::Engine::getCurrent(); # MyNews Engine

	if ($updates{$self->{mnid}} > $self->{mntm})
	{
	    $msg = MN::__("Needs update");
	    $code .= '<a href="themes.cgi?update='.$self->{mnid}.'"><div class="statusoff'.$css.'" title="'.$msg.'">'."\n";
	    $code .= "<img class=\"imgstatusoff$css\" src=\"$mne->{conf}->{weburl}/img/off.png\" alt=\"$msg\" title=\"$msg\" border=0>\n";
	    $code .= "</div></a>\n";
	}
	else
	{
	    $msg = MN::__("Up-to-date");
	    $code .= '<div class="statuson'.$css.'" title="'.$msg.'">'."\n";
	    $code .= "<img class=\"imgstatuson$css\" src=\"$mne->{conf}->{weburl}/img/on.png\" alt=\"$msg\" title=\"$msg\" border=0>\n";
	    $code .= "</div>\n";
	}
    }
    else
    {
	$code .= "&nbsp;";
    }

    return $code;
}

sub updateFromMyNews # update theme from MyNews website
{
    my $self = shift;
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{mnurl})
    {
	my $http = new HTTP::Lite();
	$http->add_req_header("Accept-Language", $self->{lang});
	$http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	my $req = $http->request("$mne->{conf}->{mnurl}/themes/update.php?id=$self->{mnid}");
	if ($req)
	{
	    my $mntfile = "/tmp/$ENV{REMOTE_USER}.mnt";
	    open(MNT, ">$mntfile");
	    print MNT $http->body();
	    close(MNT);
	    $self->updateFromFile($mntfile);
	    unlink($mntfile);

	    $http = new HTTP::Lite();
	    $http->add_req_header("Accept-Language", $self->{lang});
	    $http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	    $req = $http->request("$mne->{conf}->{mnurl}/themes/update.php?thm=$self->{mnid}");
	    if ($req)
	    {
		my $thmfile = "/tmp/$ENV{REMOTE_USER}.thm";
		open(THM, ">$thmfile");
		print THM $http->body();
		close(THM);
		$self->updateFromBin($thmfile);
		unlink($thmfile);
		$self->{mntm} = time();
	    }
	}
    }
}

sub updateFromFile # update theme from .mnt file
{
    my $self = shift;
    my $mntfile = shift; # .mnt file

    my $tid = $self->{id};
    my $line;
    my $key;
    my $val;
    my $mlf = 0;
    my $str = "";
    my $sep = "";
    open(MNT, $mntfile);
    while ($line = <MNT>)
    {
	chomp($line);
	($key,$val) = split(/\t+/, $line);
	if ($mlf)
	{
	    if ($key eq 'end')
	    {
		$self->{$mlf} = $str;
		$str = "";
		$sep = "";
		$mlf = 0;
	    }
	    else
	    {
		$str .= $sep.$line;
		$sep = "\n";
	    }
	}
	elsif ($key eq 'start')
	{
	    $mlf = $val;
	    $str = "";
	    $sep = "";
	}
	else
	{
	    $self->{$key} = $val;
	}
    }
    close(MNT);
    if ($tid)
    {
	$self->{id} = $tid;
    }
}

sub getBinCode # get text code to export a theme
{
    my $self = shift;
    
    my $code = "";

    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{webdir})
    {
	my $tar = new Archive::Tar();
	my $thdir = "$mne->{conf}->{webdir}/css/themes";
	chdir("$mne->{conf}->{webdir}/css/themes");
	$tar->add_files("$self->{id}/");
	opendir(DIR, "$thdir/$self->{id}");
	my $file;
	while ($file = readdir(DIR))
	{
	    if ($file =~ /^\./) {next;}
	    $tar->add_files("$self->{id}/$file");
	}
	closedir(DIR);
	my $thmfile = "/tmp/$self->{id}.thm";
	$tar->write($thmfile, 1);
	open(THM, $thmfile);
	my $line;
	while ($line = <THM>)
	{
	    $code .=  $line;
	}
	close(THM);
	unlink($thmfile);
    }

    return $code;
}

sub updateFromBin # update theme from .thm file
{
    my $self = shift;
    my $thmfile = shift;

    my $tar = new Archive::Tar();
    my @files = $tar->read($thmfile, 1);

    my $file;
    my $thid;
    my $filename;
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    my $thdir = "$mne->{conf}->{webdir}/css/themes/$self->{id}";

    foreach $file (@files)
    {
	if ($file->{name} =~ /([^\/]+)\/theme\.css$/)
	{
	    $thid = $1;
	    last;
	}
    }

    if ($thid)
    {
	if (!-e $thdir)
	{
	    mkdir($thdir, 0750);
	    chmod(0750, $thdir);
	}
	foreach $file (@files)
	{
	    if ($file->{name} =~ /$thid\/(.+)$/)
	    {
		$filename = $1;
		if ($filename eq "theme.css" && $thid ne $self->{id})
		{
		    $file->{data} =~ s/(\/css\/themes\/)$thid\//$1$self->{id}\//mg;
		}
		open(FILE, ">$thdir/$filename");
		print FILE $file->{data};
		close(FILE);
	    }
	}
    }
}

sub getNewThemes # get list of new themes on MyNews website
{
    my $self = shift;

    my @themes = ();
    my $theme;
    foreach $theme (keys(%updates))
    {
	if (!$current{$theme})
	{
	    push @themes, $theme;
	}
    }
    return sort @themes;
}

1;

__END__

=pod

=head1 NAME

MN::Theme - MyNews class to manage graphical themes.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 my $cgi = new MN::CGI();
 my $theme = MN::Theme->getTheme("mozilla");
 ...

=head1 DESCRIPTION

MyNews Theme objects store informations about MyNews web base interface themes.

MN::Theme inherits from MN::Object, that means that theme objects are stored in database.

=over 4

=head2 Fields

=item id : string

Theme id (primary key). For example: "mozilla".

=item name : string

Theme name (displayed in theme selector). For example: "Mozilla".

=item comment : string

Free text to describe the theme. For example: "Mozilla modern theme".

=item mnid : string

Id of the theme in the MyNews distant website (for theme update purpose).

=item mntm : string

Date of the last theme update from the MyNews distant website.

=head2 Theme information

=item getTheme ($id)

Return theme object by its id. Avoid database access if the theme object has been already selected.

=item getSelectorCode ([$selected], [$submit], [$css], [$name])

Return HTML code to display a theme selector. $selected is the default selected theme. If $submit = 1, the current form that contains the theme selector is submited when the selected theme is changed. $css is the suffix added to the CSS class for the selector. $name is the alternate parameter name of the selector ("theme" by default).

=head2 CSS

=item readCSS ([$mode])

Read the CSS file of the theme and put the code in the B<css> field.

If $mode is set to 1, read the CSS file for the printer mode.

=item writeCSS ([$mode])

Update the CSS file of the theme with the value of the B<css> field.

If $mode is set to 1, update the CSS file for the printer mode.

=head2 Theme import/export

=item getBinCode ()

Return the binary representation of a theme for exportation purpose (.thm file).

=item updateFromFile ($mntfile)

Update a theme object with the text representation in the $mntfile file.

=item updateFromBin ($thmfile)

Update a theme with the binary archive in the $tmpfile file.

=head2 Theme update from MyNews distant site

=item getMyNewsUpdates ()

Get themes update time from MyNews distant site.

=item getUpdateCode ([$css])

Get the update status of the theme (Needs update or Up-to-date). The B<getMyNewsUpdates> method must have been called before calling this method.

$css is the suffix to add to the update status CSS class.

=item updateFromMyNews ()

Update a theme from MyNews distant site. Change B<mntm> field value.

=item getNewThemes ()

Get the new themes in the MyNews distant site not present in the MyNews local site. The B<getMyNewsUpdates> method must have been called before calling this method.

=head1 SEE ALSO

L<MN::CGI>
